/* $Id: _remctlmodule.c,v 1.1 2008/02/23 19:49:24 kula Exp $
 * _remctlmodule.c: a low level interface for Python to remctl
 * 
 * Written by Thomas L. Kula <kula@tproa.net>
 */


#include <Python.h>

#include <stdlib.h>

#include <remctl.h>


    PyObject
*remctlsimple( PyObject *self, PyObject *args )
{
    struct remctl_result    *rr;
    char		    *host;
    short		    port;
    char		    *principal;
    const char		    **command;
    PyObject		    *listobj;
    PyObject		    *tmpobj;
    int			    ll;		    /* List length */
    int			    lc;		    /* List counter */
    PyObject		    *result = NULL;

    if ( PyArg_ParseTuple( args, "sHsO", &host, &port, &principal, &listobj )) {
	/* Now we have to turn listobj into an array of string pointers */

	ll = PyList_Size( listobj );
	/* malloc enough memory to hold ll + 1 char* */

	if (( command = malloc( (ll + 1) * sizeof(char *))) == NULL ) {
	    return( result );
	}
	

	for ( lc = 0; lc < ll; lc++ ) {
	    if (( tmpobj = PyList_GetItem( listobj, lc)) == NULL ) {
		return( result );
	    }

	    if (( command[ lc ] = PyString_AsString( tmpobj )) == NULL ) {
		return( result );
	    }
	}

	command[ ll ] = NULL;

	rr = remctl( host, port, principal, command );

	result = Py_BuildValue( "(ss#s#i)", rr->error,
					    rr->stdout_buf,
					    (int)rr->stdout_len,
					    rr->stderr_buf,
					    (int)rr->stderr_len,
					    rr->status );
	remctl_result_free( rr );

    }
    /* Return either the constructed result, or NULL */

    return( result );
}

PyMethodDef methods[] = {
    { "remctl", remctlsimple, METH_VARARGS},
    {NULL, NULL},
};

    void
init_remctl()
{
    (void)Py_InitModule( "_remctl", methods );
}
